import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { Combobox, ComboboxItem, ComboboxPopover, ComboboxProvider, Portal, useComboboxContext, useComboboxStore, } from "@ariakit/react";
import { cn } from "@udecode/cn";
import { filterWords } from "@udecode/plate-combobox";
import { useComboboxInput, useHTMLInputCursorState, } from "@udecode/plate-combobox/react";
import { createPointRef, getPointBefore, insertText, moveSelection, } from "@udecode/plate-common";
import { 
// @ts-ignore
findNodePath, useComposedRef, useEditorRef, } from "@udecode/plate-common/react";
import { cva } from "class-variance-authority";
import React, { createContext, forwardRef, startTransition, useCallback, useContext, useEffect, useMemo, useState, } from "react";
const InlineComboboxContext = createContext(null);
export const defaultFilter = ({ keywords = [], value }, search) => [value, ...keywords].some((keyword) => filterWords(keyword, search));
const InlineCombobox = ({ children, element, filter = defaultFilter, hideWhenNoValue = false, setValue: setValueProp, showTrigger = true, trigger, value: valueProp, }) => {
    const editor = useEditorRef();
    const inputRef = React.useRef(null);
    const cursorState = useHTMLInputCursorState(inputRef);
    const [valueState, setValueState] = useState("");
    const hasValueProp = valueProp !== undefined;
    const value = hasValueProp ? valueProp : valueState;
    const setValue = useCallback((newValue) => {
        setValueProp?.(newValue);
        if (!hasValueProp) {
            setValueState(newValue);
        }
    }, [setValueProp, hasValueProp]);
    /**
     * Track the point just before the input element so we know where to
     * insertText if the combobox closes due to a selection change.
     */
    const [insertPoint, setInsertPoint] = useState(null);
    useEffect(() => {
        const path = findNodePath(editor, element);
        if (!path)
            return;
        const point = getPointBefore(editor, path);
        if (!point)
            return;
        const pointRef = createPointRef(editor, point);
        setInsertPoint(pointRef);
        return () => {
            pointRef.unref();
        };
    }, [editor, element]);
    const { props: inputProps, removeInput } = useComboboxInput({
        cancelInputOnBlur: false,
        cursorState,
        onCancelInput: (cause) => {
            if (cause !== "backspace") {
                insertText(editor, trigger + value, {
                    at: insertPoint?.current ?? undefined,
                });
            }
            if (cause === "arrowLeft" || cause === "arrowRight") {
                moveSelection(editor, {
                    distance: 1,
                    reverse: cause === "arrowLeft",
                });
            }
        },
        ref: inputRef,
    });
    const [hasEmpty, setHasEmpty] = useState(false);
    const contextValue = useMemo(() => ({
        filter,
        inputProps,
        inputRef,
        removeInput,
        setHasEmpty,
        showTrigger,
        trigger,
    }), [
        trigger,
        showTrigger,
        filter,
        inputRef,
        inputProps,
        removeInput,
        setHasEmpty,
    ]);
    const store = useComboboxStore({
        // open: ,
        setValue: (newValue) => startTransition(() => setValue(newValue)),
    });
    const items = store.useState("items");
    /**
     * If there is no active ID and the list of items changes, select the first
     * item.
     */
    useEffect(() => {
        if (!store.getState().activeId) {
            store.setActiveId(store.first());
        }
    }, [items, store]);
    return (_jsx("span", { contentEditable: false, children: _jsx(ComboboxProvider, { open: (items.length > 0 || hasEmpty) &&
                (!hideWhenNoValue || value.length > 0), store: store, children: _jsx(InlineComboboxContext.Provider, { value: contextValue, children: children }) }) }));
};
const InlineComboboxInput = forwardRef(({ className, ...props }, propRef) => {
    const { inputProps, inputRef: contextRef, showTrigger, trigger, } = useContext(InlineComboboxContext);
    const store = useComboboxContext();
    const value = store.useState("value");
    const ref = useComposedRef(propRef, contextRef);
    /**
     * To create an auto-resizing input, we render a visually hidden span
     * containing the input value and position the input element on top of it.
     * This works well for all cases except when input exceeds the width of the
     * container.
     */
    return (_jsxs(_Fragment, { children: [showTrigger && trigger, _jsxs("span", { className: "relative min-h-[1lh]", children: [_jsx("span", { "aria-hidden": "true", className: "invisible overflow-hidden text-nowrap", children: value || "\u200B" }), _jsx(Combobox, { autoSelect: true, className: cn("absolute left-0 top-0 size-full bg-transparent outline-none", className), ref: ref, value: value, ...inputProps, ...props })] })] }));
});
InlineComboboxInput.displayName = "InlineComboboxInput";
const InlineComboboxContent = ({ className, ...props }) => {
    // Portal prevents CSS from leaking into popover
    return (_jsx(Portal, { children: _jsx(ComboboxPopover, { className: cn("z-[500] max-h-[288px] w-[300px] overflow-y-auto rounded-md bg-popover shadow-md bg-white", className), ...props }) }));
};
const comboboxItemVariants = cva("relative flex h-9 select-none items-center rounded-sm px-2 py-1.5 text-sm outline-none", {
    defaultVariants: {
        interactive: true,
    },
    variants: {
        interactive: {
            false: "",
            true: "cursor-pointer transition-colors hover:bg-accent hover:text-accent-foreground data-[active-item=true]:bg-accent data-[active-item=true]:text-accent-foreground",
        },
    },
});
const InlineComboboxItem = ({ className, keywords, onClick, ...props }) => {
    const { value } = props;
    const { filter, removeInput } = useContext(InlineComboboxContext);
    const store = useComboboxContext();
    // Optimization: Do not subscribe to value if filter is false
    const search = filter && store.useState("value");
    const visible = useMemo(() => !filter || filter({ keywords, value }, search), [filter, value, keywords, search]);
    if (!visible)
        return null;
    return (_jsx(ComboboxItem, { className: cn(comboboxItemVariants(), className), onClick: (event) => {
            removeInput(true);
            onClick?.(event);
        }, ...props }));
};
const InlineComboboxEmpty = ({ children, className, }) => {
    const { setHasEmpty } = useContext(InlineComboboxContext);
    const store = useComboboxContext();
    const items = store.useState("items");
    useEffect(() => {
        setHasEmpty(true);
        return () => {
            setHasEmpty(false);
        };
    }, [setHasEmpty]);
    if (items.length > 0)
        return null;
    return (_jsx("div", { className: cn(comboboxItemVariants({ interactive: false }), className), children: children }));
};
export { InlineCombobox, InlineComboboxContent, InlineComboboxEmpty, InlineComboboxInput, InlineComboboxItem, };
//# sourceMappingURL=inline-combobox.js.map